@@ -231,6 +231,96 @@ class GroupHelperTest extends TestCase {
 					),
 				),
 			),
+
+			// FIXME
+			// Non-local users are currently not distinguishable, so grouping them might
+			// remove the information how many different users performed the same action.
+			// So we do not group them anymore, until we found another solution.
+			array(
+				true,
+				array(
+					array(
+						'activity_id'	=> 3,
+						'user'			=> '',
+						'affecteduser'	=> 'affecteduser',
+						'app'			=> 'app1',
+						'type'			=> 'type1',
+						'subject'		=> 'subject1',
+						'subjectparams'	=> serialize(['testing/file3.txt']),
+						'message'		=> '',
+						'messageparams'	=> serialize([]),
+						'file'			=> 'testing/file3.txt',
+						'timestamp'		=> time(),
+					),
+					array(
+						'activity_id'	=> 2,
+						'user'			=> '',
+						'affecteduser'	=> 'affecteduser',
+						'app'			=> 'app1',
+						'type'			=> 'type1',
+						'subject'		=> 'subject1',
+						'subjectparams'	=> serialize(['testing/file2.txt']),
+						'message'		=> '',
+						'messageparams'	=> serialize([]),
+						'file'			=> 'testing/file2.txt',
+						'timestamp'		=> time(),
+					),
+					array(
+						'activity_id'	=> 1,
+						'user'			=> '',
+						'affecteduser'	=> 'affecteduser',
+						'app'			=> 'app1',
+						'type'			=> 'NOTtype1',
+						'subject'		=> 'NOTsubject1',
+						'subjectparams'	=> serialize(['testing/file1.txt']),
+						'message'		=> '',
+						'messageparams'	=> serialize([]),
+						'file'			=> 'testing/file1.txt',
+						'timestamp'		=> time() - 10,
+					),
+				),
+				array(
+					array(
+						'activity_id'	=> 3,
+						'user'			=> '',
+						'affecteduser'	=> 'affecteduser',
+						'app'			=> 'app1',
+						'type'			=> 'type1',
+						'subject'		=> 'subject1',
+						'subjectparams'	=> array('testing/file3.txt'),
+						'message'		=> '',
+						'messageparams'	=> array(),
+						'file'			=> 'testing/file3.txt',
+						'typeicon'		=> null,
+					),
+					array(
+						'activity_id'	=> 2,
+						'user'			=> '',
+						'affecteduser'	=> 'affecteduser',
+						'app'			=> 'app1',
+						'type'			=> 'type1',
+						'subject'		=> 'subject1',
+						'subjectparams'	=> array('testing/file2.txt'),
+						'message'		=> '',
+						'messageparams'	=> array(),
+						'file'			=> 'testing/file2.txt',
+						'typeicon'		=> null,
+					),
+					array(
+						'activity_id'	=> 1,
+						'user'			=> '',
+						'affecteduser'	=> 'affecteduser',
+						'app'			=> 'app1',
+						'type'			=> 'NOTtype1',
+						'subject'		=> 'NOTsubject1',
+						'subjectparams'	=> array('testing/file1.txt'),
+						'message'		=> '',
+						'messageparams'	=> array(),
+						'file'			=> 'testing/file1.txt',
+						'typeicon'		=> '',
+					),
+				),
+			),
 			array(
 				true,
 				array(
@@ -488,6 +578,7 @@ class GroupHelperTest extends TestCase {
 				new ParameterHelper(
 					$activityManager,
 					new \OC\Files\View(''),
+					$this->getMockBuilder('OCP\IConfig')->disableOriginalConstructor()->getMock(),
 					$activityLanguage
 				),
 				$activityLanguage
