@@ -22,6 +22,8 @@
 
 require_once __DIR__ . '/../appinfo/app.php';
 
+use OC\Files\Storage\Temporary;
+
 /**
  * Class Test_Files_versions
  * this class provide basic files versions test
@@ -70,7 +72,10 @@ class Test_Files_Versioning extends \Test\TestCase {
 	}
 
 	protected function tearDown() {
-		$this->rootView->deleteAll(self::USERS_VERSIONS_ROOT);
+		$this->rootView->deleteAll(self::TEST_VERSIONS_USER . '/files/');
+		$this->rootView->deleteAll(self::TEST_VERSIONS_USER2 . '/files/');
+		$this->rootView->deleteAll(self::TEST_VERSIONS_USER . '/files_versions/');
+		$this->rootView->deleteAll(self::TEST_VERSIONS_USER2 . '/files_versions/');
 
 		parent::tearDown();
 	}
@@ -248,9 +253,6 @@ class Test_Files_Versioning extends \Test\TestCase {
 
 		$this->assertTrue($this->rootView->file_exists($v1Renamed));
 		$this->assertTrue($this->rootView->file_exists($v2Renamed));
-
-		//cleanup
-		\OC\Files\Filesystem::unlink('test2.txt');
 	}
 
 	public function testRenameInSharedFolder() {
@@ -292,9 +294,131 @@ class Test_Files_Versioning extends \Test\TestCase {
 
 		$this->assertTrue($this->rootView->file_exists($v1Renamed));
 		$this->assertTrue($this->rootView->file_exists($v2Renamed));
+	}
 
-		//cleanup
-		\OC\Files\Filesystem::unlink('/folder1/folder2/test.txt');
+	public function testMoveFolder() {
+
+		\OC\Files\Filesystem::mkdir('folder1');
+		\OC\Files\Filesystem::mkdir('folder2');
+		\OC\Files\Filesystem::file_put_contents('folder1/test.txt', 'test file');
+
+		$t1 = time();
+		// second version is two weeks older, this way we make sure that no
+		// version will be expired
+		$t2 = $t1 - 60 * 60 * 24 * 14;
+
+		// create some versions
+		$this->rootView->mkdir(self::USERS_VERSIONS_ROOT . '/folder1');
+		$v1 = self::USERS_VERSIONS_ROOT . '/folder1/test.txt.v' . $t1;
+		$v2 = self::USERS_VERSIONS_ROOT . '/folder1/test.txt.v' . $t2;
+		$v1Renamed = self::USERS_VERSIONS_ROOT . '/folder2/folder1/test.txt.v' . $t1;
+		$v2Renamed = self::USERS_VERSIONS_ROOT . '/folder2/folder1/test.txt.v' . $t2;
+
+		$this->rootView->file_put_contents($v1, 'version1');
+		$this->rootView->file_put_contents($v2, 'version2');
+
+		// execute rename hook of versions app
+		\OC\Files\Filesystem::rename('folder1', 'folder2/folder1');
+
+		$this->assertFalse($this->rootView->file_exists($v1));
+		$this->assertFalse($this->rootView->file_exists($v2));
+
+		$this->assertTrue($this->rootView->file_exists($v1Renamed));
+		$this->assertTrue($this->rootView->file_exists($v2Renamed));
+	}
+
+
+	public function testMoveFileIntoSharedFolderAsRecipient() {
+
+		\OC\Files\Filesystem::mkdir('folder1');
+		$fileInfo = \OC\Files\Filesystem::getFileInfo('folder1');
+
+		\OCP\Share::shareItem(
+			'folder',
+			$fileInfo['fileid'],
+			\OCP\Share::SHARE_TYPE_USER,
+			self::TEST_VERSIONS_USER2,
+			\OCP\Constants::PERMISSION_ALL
+		);
+
+		self::loginHelper(self::TEST_VERSIONS_USER2);
+		$versionsFolder2 = '/' . self::TEST_VERSIONS_USER2 . '/files_versions';
+		\OC\Files\Filesystem::file_put_contents('test.txt', 'test file');
+
+		$t1 = time();
+		// second version is two weeks older, this way we make sure that no
+		// version will be expired
+		$t2 = $t1 - 60 * 60 * 24 * 14;
+
+		$this->rootView->mkdir($versionsFolder2);
+		// create some versions
+		$v1 = $versionsFolder2 . '/test.txt.v' . $t1;
+		$v2 = $versionsFolder2 . '/test.txt.v' . $t2;
+
+		$this->rootView->file_put_contents($v1, 'version1');
+		$this->rootView->file_put_contents($v2, 'version2');
+
+		// move file into the shared folder as recipient
+		\OC\Files\Filesystem::rename('/test.txt', '/folder1/test.txt');
+
+		$this->assertFalse($this->rootView->file_exists($v1));
+		$this->assertFalse($this->rootView->file_exists($v2));
+
+		self::loginHelper(self::TEST_VERSIONS_USER);
+
+		$versionsFolder1 = '/' . self::TEST_VERSIONS_USER . '/files_versions';
+
+		$v1Renamed = $versionsFolder1 . '/folder1/test.txt.v' . $t1;
+		$v2Renamed = $versionsFolder1 . '/folder1/test.txt.v' . $t2;
+
+		$this->assertTrue($this->rootView->file_exists($v1Renamed));
+		$this->assertTrue($this->rootView->file_exists($v2Renamed));
+	}
+
+	public function testMoveFolderIntoSharedFolderAsRecipient() {
+
+		\OC\Files\Filesystem::mkdir('folder1');
+		$fileInfo = \OC\Files\Filesystem::getFileInfo('folder1');
+
+		\OCP\Share::shareItem(
+			'folder',
+			$fileInfo['fileid'],
+			\OCP\Share::SHARE_TYPE_USER,
+			self::TEST_VERSIONS_USER2,
+			\OCP\Constants::PERMISSION_ALL
+		);
+
+		self::loginHelper(self::TEST_VERSIONS_USER2);
+		$versionsFolder2 = '/' . self::TEST_VERSIONS_USER2 . '/files_versions';
+		\OC\Files\Filesystem::mkdir('folder2');
+		\OC\Files\Filesystem::file_put_contents('folder2/test.txt', 'test file');
+
+		$t1 = time();
+		// second version is two weeks older, this way we make sure that no
+		// version will be expired
+		$t2 = $t1 - 60 * 60 * 24 * 14;
+
+		$this->rootView->mkdir($versionsFolder2);
+		$this->rootView->mkdir($versionsFolder2 . '/folder2');
+		// create some versions
+		$v1 = $versionsFolder2 . '/folder2/test.txt.v' . $t1;
+		$v2 = $versionsFolder2 . '/folder2/test.txt.v' . $t2;
+
+		$this->rootView->file_put_contents($v1, 'version1');
+		$this->rootView->file_put_contents($v2, 'version2');
+
+		// move file into the shared folder as recipient
+		\OC\Files\Filesystem::rename('/folder2', '/folder1/folder2');
+
+		self::loginHelper(self::TEST_VERSIONS_USER);
+
+		$versionsFolder1 = '/' . self::TEST_VERSIONS_USER . '/files_versions';
+
+		$v1Renamed = $versionsFolder1 . '/folder1/folder2/test.txt.v' . $t1;
+		$v2Renamed = $versionsFolder1 . '/folder1/folder2/test.txt.v' . $t2;
+
+		$this->assertTrue($this->rootView->file_exists($v1Renamed));
+		$this->assertTrue($this->rootView->file_exists($v2Renamed));
 	}
 
 	public function testRenameSharedFile() {
@@ -335,9 +459,6 @@ class Test_Files_Versioning extends \Test\TestCase {
 
 		$this->assertFalse($this->rootView->file_exists($v1Renamed));
 		$this->assertFalse($this->rootView->file_exists($v2Renamed));
-
-		//cleanup
-		\OC\Files\Filesystem::unlink('/test.txt');
 	}
 
 	public function testCopy() {
@@ -366,10 +487,6 @@ class Test_Files_Versioning extends \Test\TestCase {
 
 		$this->assertTrue($this->rootView->file_exists($v1Copied));
 		$this->assertTrue($this->rootView->file_exists($v2Copied));
-
-		//cleanup
-		\OC\Files\Filesystem::unlink('test.txt');
-		\OC\Files\Filesystem::unlink('test2.txt');
 	}
 
 	/**
@@ -406,6 +523,100 @@ class Test_Files_Versioning extends \Test\TestCase {
 		$this->rootView->deleteAll(self::USERS_VERSIONS_ROOT . '/subfolder');
 	}
 
+	public function testRestoreSameStorage() {
+		\OC\Files\Filesystem::mkdir('sub');
+		$this->doTestRestore();
+	}
+
+	public function testRestoreCrossStorage() {
+		$storage2 = new Temporary(array());
+		\OC\Files\Filesystem::mount($storage2, array(), self::TEST_VERSIONS_USER . '/files/sub');
+
+		$this->doTestRestore();
+	}
+
+	private function doTestRestore() {
+		$filePath = self::TEST_VERSIONS_USER . '/files/sub/test.txt';
+		$this->rootView->file_put_contents($filePath, 'test file');
+
+		$t0 = $this->rootView->filemtime($filePath);
+
+		// not exactly the same timestamp as the file
+		$t1 = time() - 60;
+		// second version is two weeks older
+		$t2 = $t1 - 60 * 60 * 24 * 14;
+
+		// create some versions
+		$v1 = self::USERS_VERSIONS_ROOT . '/sub/test.txt.v' . $t1;
+		$v2 = self::USERS_VERSIONS_ROOT . '/sub/test.txt.v' . $t2;
+
+		$this->rootView->mkdir(self::USERS_VERSIONS_ROOT . '/sub');
+		$this->rootView->file_put_contents($v1, 'version1');
+		$this->rootView->file_put_contents($v2, 'version2');
+
+		$oldVersions = \OCA\Files_Versions\Storage::getVersions(
+			self::TEST_VERSIONS_USER, '/sub/test.txt'
+		);
+
+		$this->assertCount(2, $oldVersions);
+
+		$this->assertEquals('test file', $this->rootView->file_get_contents($filePath));
+		$info1 = $this->rootView->getFileInfo($filePath);
+
+		\OCA\Files_Versions\Storage::rollback('sub/test.txt', $t2);
+
+		$this->assertEquals('version2', $this->rootView->file_get_contents($filePath));
+		$info2 = $this->rootView->getFileInfo($filePath);
+
+		$this->assertNotEquals(
+			$info2['etag'],
+			$info1['etag'],
+			'Etag must change after rolling back version'
+		);
+		$this->assertEquals(
+			$info2['fileid'],
+			$info1['fileid'],
+			'File id must not change after rolling back version'
+		);
+		$this->assertEquals(
+			$info2['mtime'],
+			$t2,
+			'Restored file has mtime from version'
+		);
+
+		$newVersions = \OCA\Files_Versions\Storage::getVersions(
+			self::TEST_VERSIONS_USER, '/sub/test.txt'
+		);
+
+		$this->assertTrue(
+			$this->rootView->file_exists(self::USERS_VERSIONS_ROOT . '/sub/test.txt.v' . $t0),
+			'A version file was created for the file before restoration'
+		);
+		$this->assertTrue(
+			$this->rootView->file_exists($v1),
+			'Untouched version file is still there'
+		);
+		$this->assertFalse(
+			$this->rootView->file_exists($v2),
+			'Restored version file gone from files_version folder'
+		);
+
+		$this->assertCount(2, $newVersions, 'Additional version created');
+
+		$this->assertTrue(
+			isset($newVersions[$t0 . '#' . 'test.txt']),
+			'A version was created for the file before restoration'
+		);
+		$this->assertTrue(
+			isset($newVersions[$t1 . '#' . 'test.txt']),
+			'Untouched version is still there'
+		);
+		$this->assertFalse(
+			isset($newVersions[$t2 . '#' . 'test.txt']),
+			'Restored version is not in the list any more'
+		);
+	}
+
 	/**
 	 * @param string $user
 	 * @param bool $create
