@@ -1,9 +1,16 @@
-<?php $TRANSLATIONS = array(
+<?php
+$TRANSLATIONS = array(
 "Pictures" => "Imagens",
 "Picture view" => "Visualizar foto",
+"Next" => "Próxima",
+"Play" => "Reproduzir",
+"Pause" => "Pausa",
+"Previous" => "Anterior",
+"Close" => "Fechar",
 "Error loading slideshow template" => "Erro ao carregar o modelo de slideshow",
 "Share" => "Compartilhar",
 "No pictures found! If you upload pictures in the files app, they will be displayed here." => "Nenhuma foto foi encontrada! Se você enviar fotos no app arquivos, eles serão exibidos aqui.",
 "shared by %s" => "compartilhado por %s",
 "File list" => "Lista de arquivo"
 );
+$PLURAL_FORMS = "nplurals=2; plural=(n > 1);";
