@@ -85,6 +85,13 @@ class OfflineUser {
 		$this->fetchDetails();
 	}
 
+	/**
+	 * remove the Delete-flag from the user.
+	 */
+	public function unmark() {
+		$this->config->setUserValue($this->ocName, 'user_ldap', 'isDeleted', '0');
+	}
+
 	/**
 	 * exports the user details in an assoc array
 	 * @return array
