@@ -24,11 +24,13 @@
 
 namespace OC\AppFramework\Utility;
 
+use \OCP\AppFramework\Utility\IControllerMethodReflector;
+
 
 /**
  * Reads and parses annotations from doc comments
  */
-class ControllerMethodReflector {
+class ControllerMethodReflector implements IControllerMethodReflector{
 
 	private $annotations;
 	private $types;
@@ -62,6 +64,7 @@ class ControllerMethodReflector {
 		} else {
 			$this->types = array_combine($matches['var'], $matches['type']);
 		}
+
 		// get method parameters
 		foreach ($reflection->getParameters() as $param) {
 			if($param->isOptional()) {
