@@ -45,7 +45,7 @@ class OC_Connector_Sabre_MaintenancePlugin extends \Sabre\DAV\ServerPlugin
 	 * @return bool
 	 */
 	public function checkMaintenanceMode() {
-		if (\OCP\Config::getSystemValue('singleuser', false)) {
+		if (\OC::$server->getSystemConfig()->getValue('singleuser', false)) {
 			throw new \Sabre\DAV\Exception\ServiceUnavailable();
 		}
 		if (OC_Config::getValue('maintenance', false)) {
