@@ -91,6 +91,10 @@ class OC_Installer{
 		}
 
 		$extractDir .= '/' . $info['id'];
+		if(!file_exists($extractDir)) {
+			OC_Helper::rmdirr($basedir);
+			throw new \Exception($l->t("Archive does not contain a directory named %s", $info['id']));
+		}
 		OC_Helper::copyr($extractDir, $basedir);
 
 		//remove temporary files
@@ -205,8 +209,8 @@ class OC_Installer{
 	 * @throws Exception
 	 */
 	public static function updateAppByOCSId($ocsid) {
-		$appdata = OC_OCSClient::getApplication($ocsid);
-		$download = OC_OCSClient::getApplicationDownload($ocsid, 1);
+		$appdata = OC_OCSClient::getApplication($ocsid, \OC_Util::getVersion());
+		$download = OC_OCSClient::getApplicationDownload($ocsid, 1, \OC_Util::getVersion());
 
 		if (isset($download['downloadlink']) && trim($download['downloadlink']) !== '') {
 			$download['downloadlink'] = str_replace(' ', '%20', $download['downloadlink']);
@@ -368,7 +372,7 @@ class OC_Installer{
 
 		if($ocsid<>'') {
 
-			$ocsdata=OC_OCSClient::getApplication($ocsid);
+			$ocsdata=OC_OCSClient::getApplication($ocsid, \OC_Util::getVersion());
 			$ocsversion= (string) $ocsdata['version'];
 			$currentversion=OC_App::getAppVersion($app);
 			if (version_compare($ocsversion, $currentversion, '>')) {
