@@ -9,11 +9,16 @@
 namespace OC\Preview;
 
 class Image extends Provider {
-
+	/**
+	 * {@inheritDoc}
+	 */
 	public function getMimeType() {
 		return '/image\/(?!tiff$)(?!svg.*).*/';
 	}
 
+	/**
+	 * {@inheritDoc}
+	 */
 	public function getThumbnail($path, $maxX, $maxY, $scalingup, $fileview) {
 		//get fileinfo
 		$fileInfo = $fileview->getFileInfo($path);
@@ -30,16 +35,19 @@ class Image extends Provider {
 
 		$image = new \OC_Image();
 
-		if($fileInfo['encrypted'] === true) {
+		$useTempFile = $fileInfo->isEncrypted() || !$fileInfo->getStorage()->isLocal();
+		if ($useTempFile) {
 			$fileName = $fileview->toTmpFile($path);
 		} else {
 			$fileName = $fileview->getLocalFile($path);
 		}
 		$image->loadFromFile($fileName);
+		if ($useTempFile) {
+			unlink($fileName);
+		}
+		$image->fixOrientation();
 
 		return $image->valid() ? $image : false;
 	}
 
 }
-
-\OC\Preview::registerProvider('OC\Preview\Image');
