@@ -1,19 +1,17 @@
 <?php
 
-// Init owncloud
-require_once('../../lib/base.php');
-
-$l=OC_L10N::get('settings');
+$l = \OC::$server->getL10N('settings');
 
 OC_JSON::checkLoggedIn();
 OCP\JSON::callCheck();
 
+
 // Get data
-if( isset( $_POST['lang'] ) ){
+if( isset( $_POST['lang'] ) ) {
 	$languageCodes=OC_L10N::findAvailableLanguages();
 	$lang=$_POST['lang'];
-	if(array_search($lang,$languageCodes) or $lang=='en'){
-		OC_Preferences::setValue( OC_User::getUser(), 'core', 'lang', $lang );
+	if(array_search($lang, $languageCodes) or $lang === 'en') {
+		\OC::$server->getConfig()->setUserValue( OC_User::getUser(), 'core', 'lang', $lang );
 		OC_JSON::success(array("data" => array( "message" => $l->t("Language changed") )));
 	}else{
 		OC_JSON::error(array("data" => array( "message" => $l->t("Invalid request") )));
@@ -21,5 +19,3 @@ if( isset( $_POST['lang'] ) ){
 }else{
 	OC_JSON::error(array("data" => array( "message" => $l->t("Invalid request") )));
 }
-
-?>
